﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HeuristicLab.Analysis.SolutionCaching;
using HeuristicLab.Analysis.SolutionCaching.RealVectorEncoding;
using HeuristicLab.Encodings.RealVectorEncoding;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace AlgorithmBehaviorUnitTests {
  [TestClass]
  public class RealVectorSolutionCacheTest {
    [TestMethod]
    public void TestCache() {
      var rand = new HeuristicLab.Random.FastRandom();
      int nrOfVectors = 200;
      int vectorLength = 50;

      RealVectorSolutionCache solutionCache = new RealVectorSolutionCache();
      RealVectorEqualityComparer comp = new RealVectorEqualityComparer();

      for (int i = 0; i < nrOfVectors; i++) {
        RealVector vector = new RealVector(vectorLength, rand, -20, 30);
        RealVectorSolutionInformation info = new RealVectorSolutionInformation();

        //should be added
        info.ProducedBy = ProducedBy.Crossover;
        solutionCache.Add(vector, info);

        //should be added
        info = new RealVectorSolutionInformation();
        info.ProducedBy = ProducedBy.Move;
        solutionCache.Add(vector, info);

        //should be added
        info = new RealVectorSolutionInformation();
        info.ProducedBy = ProducedBy.Move;
        info.Iteration = 20;
        solutionCache.Add(vector, info);

        //should not be added
        info = new RealVectorSolutionInformation();
        info.ProducedBy = ProducedBy.Crossover;
        solutionCache.Add(vector, info);
      }

      foreach (var value in solutionCache.Values()) {
        Assert.AreEqual(3, value.Count);
      }

      foreach (var key in solutionCache.Keys()) {
        int cnt = solutionCache.Keys().Count(x => comp.Equals(key, x));
        Assert.AreEqual(1, cnt);
      }
    }
  }
}
